#include <iostream>
#include <string>
#include <set>
#include <tuple>
#include <assert.h>
#include <time.h>
#include <unistd.h>

using namespace std;

string x_i_j(const char *s, size_t i, size_t j) {
  return string(s) + "_" + to_string(i) + "_" + to_string(j);
}

void genTest1(size_t k, bool cyclic, bool countingSuccessors) {
  assert(k > 0);

  float d1, d2;
  int dra, dsa, drb, dsb;

  d1 = (rand() % 100 + 1) / 100.0;
  do {
    d2 = (rand() % 100 + 1) / 100.0;
  } while (d2 == d1);

  dra = rand() % k + 1;
  dsa = rand() % k + 1;
  drb = rand() % k + 1;
  dsb = rand() % k + 1;

  for (size_t i = 0; i < k; i++) {
    size_t ni = (i + 1 < k ? i + 1 : 0);

    for (size_t j = 0; j < k; j++) {
      cout << x_i_j("a", i, j) << " p " << d1 << endl;
      cout << x_i_j("b", i, j) << " p " << d2 << endl;
      if (cyclic && i == 0) {
        cout << x_i_j("a", i, j) << " q " << 1 << endl;
        cout << x_i_j("b", i, j) << " q " << 1 << endl;
      }

      size_t taa = rand() % k;
      size_t tab = rand() % k;
      size_t tba = rand() % k;
      size_t tbb = rand() % k;

      if (i + 1 < k || cyclic) {
        for (size_t jp = 0; jp < k; jp++) {
          size_t rdr, rds;
          if (!countingSuccessors)
            rdr = rand(), rds = rand();
          else
            rdr = 1, rds = 1;

          if (jp == taa) {
            cout << x_i_j("a", i, j) << " " << x_i_j("a", ni, jp)
                 << " r " << 1.0 * dra / k << endl;
            cout << x_i_j("a", i, j) << " " << x_i_j("a", ni, jp)
                 << " s " << 1.0 * dsa / k << endl;
          }
          else {
            cout << x_i_j("a", i, j) << " " << x_i_j("a", ni, jp)
                 << " r " << 1.0 * (dra - rdr % dra) / k << endl;
            cout << x_i_j("a", i, j) << " " << x_i_j("a", ni, jp)
                 << " s " << 1.0 * (dsa - rds % dsa) / k << endl;
          }

          if (jp == tab) {
            cout << x_i_j("a", i, j) << " " << x_i_j("b", ni, jp)
                 << " r " << 1.0 * drb / k << endl;
            cout << x_i_j("a", i, j) << " " << x_i_j("b", ni, jp)
                 << " s " << 1.0 * dsb / k << endl;
          }
          else {
            cout << x_i_j("a", i, j) << " " << x_i_j("b", ni, jp)
                 << " r " << 1.0 * (drb - rdr % drb) / k << endl;
            cout << x_i_j("a", i, j) << " " << x_i_j("b", ni, jp)
                 << " s " << 1.0 * (dsb - rds % dsb) / k << endl;
          }

          if (jp == tba) {
            cout << x_i_j("b", i, j) << " " << x_i_j("a", ni, jp)
                 << " r " << 1.0 * dra / k << endl;
            cout << x_i_j("b", i, j) << " " << x_i_j("a", ni, jp)
                 << " s " << 1.0 * dsa / k << endl;
          }
          else {
            cout << x_i_j("b", i, j) << " " << x_i_j("a", ni, jp)
                 << " r " << 1.0 * (dra - rdr % dra) / k << endl;
            cout << x_i_j("b", i, j) << " " << x_i_j("a", ni, jp)
                 << " s " << 1.0 * (dsa - rds % dsa) / k << endl;
          }

          if (jp == tbb) {
            cout << x_i_j("b", i, j) << " " << x_i_j("b", ni, jp)
                 << " r " << 1.0 * drb / k << endl;
            cout << x_i_j("b", i, j) << " " << x_i_j("b", ni, jp)
                 << " s " << 1.0 * dsb / k << endl;
          }
          else {
            cout << x_i_j("b", i, j) << " " << x_i_j("b", ni, jp)
                 << " r " << 1.0 * (drb - rdr % drb) / k << endl;
            cout << x_i_j("b", i, j) << " " << x_i_j("b", ni, jp)
                 << " s " << 1.0 * (dsb - rds % dsb) / k << endl;
          }
        } // for jp
      } // if
    } // for j
  } // for i
}

void genTest2(size_t k, bool countingSuccessors) {
  assert(k > 0);

  for (size_t i = 0; i < k; i++) {
    size_t ni = (i + 1 < k ? i + 1 : 0);
    for (size_t j = 0; j < k; j++) {
      const char* ab[] = {"a", "b"};
      for (auto c: ab) {
        if (j == 0)
          cout << x_i_j(c, i, j) << " p " << 1 << endl;

        size_t nj = (j + 1 < k ? j + 1 : 0);
        for (size_t ip = 0; ip < k; ip++) {
          if ((ip == i && j < k - 1) || (ip == ni && j == k - 1)) {
            cout << x_i_j(c, i, j) << " " << x_i_j(c, ip, nj)
                 << " r " << 1 << endl;
            cout << x_i_j(c, i, j) << " " << x_i_j(c, ip, nj)
                 << " s " << 1 << endl;
          }
          else if (!countingSuccessors) {
            cout << x_i_j(c, i, j) << " " << x_i_j(c, ip, nj)
                 << " r " << 1.0 * (rand() % k + 1) / k << endl;
            cout << x_i_j(c, i, j) << " " << x_i_j(c, ip, nj)
                 << " s " << 1.0 * (rand() % k + 1) / k << endl;
          }
          else {
            cout << x_i_j(c, i, j) << " " << x_i_j(c, ip, nj)
                 << " r " << 1.0 / k << endl;
            cout << x_i_j(c, i, j) << " " << x_i_j(c, ip, nj)
                 << " s " << 1.0 / k << endl;
          } // else
        } // for ip
      } // for c
    } // for j
  } // for i
}

void genTest3(int k, int n, int m, int p, int l, int nSV, int nSE) {
  assert(l >= 2);
  for (int i = 0; i < k; i++) {
    set<pair<size_t, size_t>> vertexLabels;
    while (vertexLabels.size() < size_t(p)) {
      int x = i*n + rand() % n;
      int q = rand() % nSV;
      vertexLabels.insert(std::make_pair(x,q));
    }
    for (auto &it: vertexLabels) {
      float d = (rand() % l + 1) * 1.0 / l;
      cout << to_string(it.first) << " " << to_string(it.second)
           << " " << d << endl;
    }

    set<tuple<size_t, size_t, size_t>> edges;
    while (edges.size() < size_t(m)) {
      int x = i*n + rand() % n;
      int y = i*n + rand() % n;
      int r = rand() % nSE;
      edges.insert(std::make_tuple(x,y,r));
    }
    for (auto &it: edges) {
      float d = (rand() % l + 1) * 1.0 / l;
      cout << to_string(get<0>(it)) << " "
           << to_string(get<1>(it)) << " "
           << to_string(get<2>(it)) << " "
           << d << endl;
    }
  }
}

// Robert Jenkins' 96 bit Mix Function
unsigned long mix(unsigned long a, unsigned long b, unsigned long c)
{
    a=a-b;  a=a-c;  a=a^(c >> 13);
    b=b-c;  b=b-a;  b=b^(a << 8);
    c=c-a;  c=c-b;  c=c^(b >> 13);
    a=a-b;  a=a-c;  a=a^(c >> 12);
    b=b-c;  b=b-a;  b=b^(a << 16);
    c=c-a;  c=c-b;  c=c^(b >> 5);
    a=a-b;  a=a-c;  a=a^(c >> 3);
    b=b-c;  b=b-a;  b=b^(a << 10);
    c=c-a;  c=c-b;  c=c^(b >> 15);
    return c;
}

int main(int argc, char* argv[])
{
  unsigned long seed = mix(clock(), time(NULL), getpid());
  srand(seed);

  assert(argc >= 2);
  int test = atoi(argv[1]);
  assert(test != 1 || argc == 5);
  assert(test != 2 || argc == 4);
  assert(test != 3 || argc == 9);

  if (test == 1) {
    size_t k = size_t(atoi(argv[2]));
    bool cyclic = atoi(argv[3]);
    bool countingSuccessors = atoi(argv[4]);
    genTest1(k, cyclic, countingSuccessors);
  }
  else if (test == 2) {
    size_t k = size_t(atoi(argv[2]));
    bool countingSuccessors = atoi(argv[3]);
    genTest2(k, countingSuccessors);
  }
  else if (test == 3) {
    int k = atoi(argv[2]);
    int n = atoi(argv[3]);
    int m = atoi(argv[4]);
    int p = atoi(argv[5]);
    int l = atoi(argv[6]);
    int nSV = atoi(argv[7]);
    int nSE = atoi(argv[8]);
    genTest3(k, n, m, p, l, nSV, nSE);
  }
  else
    assert(false);

  return 0;
}

