#ifndef LABELS_H_INCLUDED
#define LABELS_H_INCLUDED

#include <map>
#include <string>
#include <cstdint>
#include "../lanTools.h"

namespace crispBis {

template <class T>
struct Collection {
  private:
    std::map<std::string, T*> nameToPtr;

  public:
    std::map<T*, std::string> ptrToName;

    T* getPtr(const std::string name) {
      if (nameToPtr.find(name) == nameToPtr.end()) {
        T *p = NEW(T, T());
        nameToPtr[name] = p;
        ptrToName[p] = name;
        return p;
      }

      return nameToPtr[name];
    };

    std::string getName(T *p) {
      assert(ptrToName.find(p) != ptrToName.end());
      return ptrToName[p];
    };

    ~Collection() {
      for (auto &x: ptrToName) {
        DELETE(x.first);
      }
    }
};

struct IdCollection {
  private:
    std::map<std::string, size_t> nameToID;
    size_t nextID = 0;

  public:
    std::map<size_t, std::string> idToName;

    size_t getID(const std::string name) {
      if (nameToID.find(name) == nameToID.end()) {
        size_t id = nextID++;
        nameToID[name] = id;
        idToName[id] = name;
        return id;
      }

      return nameToID[name];
    };

    std::string getName(const size_t id) {
      assert(idToName.find(id) != idToName.end());
      return idToName[id];
    };
};

} // namespace crispBis

#endif // LABELS_H_INCLUDED
