#ifndef FUZZYGRAPH_H
#define FUZZYGRAPH_H

#include <stdexcept>
#include "Collection.h"
#include "DLList.h"
#include "Partition.h"

namespace crispBis {

struct Vertex;
struct Edge;
struct BlockEdge;
struct Block;
struct Partition;
struct SuperBlock;
struct SuperPartition;

struct CrispBis
{
  friend struct Test;

  IdCollection vertexLabels;
  IdCollection edgeLabels;
  Collection<Vertex> vertices;
  DLList<BlockEdge> *emptyBlockEdges;
  DLList<BlockEdge> *nonemptyBlockEdges;
  Partition P;
  std::map<size_t, SuperPartition*> Q;
  bool verbose;

  CrispBis(bool verbose);
  virtual ~CrispBis();

  void compCB();
  virtual void read(const char *filename = nullptr);
  static void main(bool verbose);

  private:
    void initialize();
    void split(Block *X, SuperBlock *Y, size_t r);
    void computeBlockEdges(std::vector<Vertex*> &vertices_of_X, size_t r);
    void computeSubblocks(std::vector<Vertex*> &vertices_of_X, size_t r);
    void doSplitting(Block *X, std::vector<Vertex*> &vertices_of_X,
                     SuperBlock *Y, size_t r);
    void clearAuxiliaryInfo(std::vector<Vertex*> &vertices_of_X, size_t r);

    void addVertexLabel(const std::string ID, const std::string p, float d);
    void addEdge(const std::string xID, const std::string yID,
                 const std::string r, float d);
    void printGraph();
};

} // namespace crispBis

#endif // FUZZYGRAPH_H
