#ifndef DLLIST_H
#define DLLIST_H

#include <string>
#include <sstream>
#include "../lanTools.h"

namespace crispBis {

// doubly linked list
template <class T>
struct DLList
{
  bool delElementsAtTheEnd;
  T *head;
  size_t size;

  DLList(bool delElementsAtTheEnd) {
    this->delElementsAtTheEnd = delElementsAtTheEnd;
    head = nullptr;
    size = 0;
  }

  ~DLList() {
    if (delElementsAtTheEnd)
      clear();
  }

  void clear() {
    for (T *w = head; w != nullptr; ) {
      T *u = w;
      w = w->next;
      DELETE(u);
    }
    head = nullptr;
    size = 0;
  }

  void add(T *elem) {
    elem->prev = nullptr;
    elem->next = head;
    if (head)
      head->prev = elem;
    head = elem;
    size++;
  }

  void remove(T *elem) {
    T *p = elem->prev;
    T *n = elem->next;
    if (p)
      p->next = n;
    if (n)
      n->prev = p;
    elem->prev = nullptr;
    elem->next = nullptr;
    if (head == elem)
      head = n;
    size--;
  }

  bool empty() { return size == 0; };

  std::string str() {
    std::stringstream buf;
    buf << "[";
    bool first = true;
    for (T* x = head; x != nullptr; x = x->next) {
      if (!first)
        buf << ", ";
      first = false;
      buf << x->str();
    }
    buf << "]";
    return buf.str();
  }
};

} // namespace crispBis

#endif // DLLIST_H
