#ifndef LANTOOLS_H_INCLUDED
#define LANTOOLS_H_INCLUDED

#include <stdlib.h>
#include <iostream>
#include <map>
#include <utility>

namespace lanTools {

#define MSG(msg) { std::cout << msg << "\n"; }
#define ERROR_MSG(msg) { std::cerr << msg << "\n"; }

#define ASSERT(cond) { \
  if (!(cond)) { \
    std::cerr << "Assertion failed at line " << __LINE__ \
              << " in file " << __FILE__ << "." << std::endl; \
    exit(-1); \
  } \
}

#ifdef DEBUG

#define assert(cond) { \
  if (!(cond)) { \
    std::cerr << "Assertion failed at line " << __LINE__ \
              << " in file " << __FILE__ << "." << std::endl; \
    exit(-1); \
  } \
}

#define DEBUG_MARK() { \
  std::cout << "DEBUG POSITION: " << __FILE__ << " " << __LINE__ << ".\n\n"; \
}

#define DEBUG_MSG(msg) { \
  std::cout << __FILE__ << " " << __LINE__ << ": " << msg << "\n\n"; \
}

#else

#define assert(cond) {}
#define DEBUG_MARK() {}
#define DEBUG_MSG(msg) {}

#endif

#ifdef VERBOSE
#define VERBOSE_MSG(msg) { std::cout << msg << "\n"; }
#else
#define VERBOSE_MSG(msg) {}
#endif

#define UNSUPPORTED() { \
  std::cerr << "\nCalling an unsupported method/function in file " \
            << __FILE__ << " at line " << __LINE__ << ".\n"; \
  exit(-1); \
}

#define TO_BE_IMPLEMENTED() { \
  std::cerr << "\nTO BE IMPLEMENTED: " << __FILE__ \
            << " " << __LINE__ << std::endl; \
  exit(-1); \
}

//------------------------------------------------------------------------------

#ifndef DEBUG_MEM
#define NEW(x,y) (new y)
#define DELETE(x) (delete x)
#define DELETE_ARRAY(x) (delete [] x)
#define CHECK_MEMORY() {}
#else
extern void *allocated_temp;
extern std::map<void*,std::pair<const char*,int>> allocated;
#define NEW(x,y) ( \
  lanTools::allocated[lanTools::allocated_temp = ((void*) new y)] = \
    std::make_pair(__FILE__,__LINE__), (x*) lanTools::allocated_temp \
)
#define DELETE(x) {lanTools::allocated.erase((void*) x); delete x;}
/*
#define DELETE(x) { \
  std::cerr << "Freeing for " << lanTools::allocated[x].first \
           << " " << lanTools::allocated[x].second << "\n"; \
  lanTools::allocated.erase((void*) x); \
  delete x; \
  std::cerr << "Done freeing." << "\n"; \
}
*/
#define DELETE_ARRAY(x) { lanTools::allocated.erase((void*) x); delete [] x;}
void print_unfreed_memory();
#define CHECK_MEMORY() { lanTools::print_unfreed_memory(); }
#endif // DEBUG_MEM

} // namespace lanTools

#endif // LANTOOLS_H_INCLUDED
