#include "Block.h"
#include "Partition.h"
#include "Vertex.h"
#include "SuperBlock.h"
#include "../lanTools.h"
#include <sstream>
#include <set>

namespace crispBis {

Block::Block(DLList<Vertex> *vertices, Partition *partition,
             std::map<size_t, SuperBlock*> &superBlocks) {
  this->vertices = vertices;
  this->partition = partition;
  partition->push_back(this);

  Vertex *x = vertices->head;
  while (x) {
    x->block = this;
    x = x->next;
  }

  for (auto &x: superBlocks) {
    size_t r = x.first;
    this->superBlocks[r] = x.second;
    x.second->addBlock(this);
  }
}

Block::~Block() {
  DELETE(vertices);
}

void Block::createBlock(DLList<Vertex> *vl) {
  assert(vl->size > 0);
  Vertex *x = vl->head;
  Block *bx = x->block;
  NEW(Block, Block(vl, bx->partition, bx->superBlocks));
}

std::string Block::str() {
  std::set<std::string> _vertices;
  Vertex *x = vertices->head;
  while (x) {
    _vertices.insert(x->getID());
    x = x->next;
  }

  std::stringstream buf;
  bool first = true;
  for (const std::string &id: _vertices) {
    if (!first)
      buf << " ";
    first = false;
    buf << id;
  }
  return buf.str();
}

std::string Block::getName() {
  std::set<std::string> _vertices;
  Vertex *x = vertices->head;
  while (x) {
    _vertices.insert(x->getID());
    x = x->next;
  }

  std::stringstream buf;
  bool first = true;
  for (const std::string &id: _vertices) {
    if (!first)
      buf << "_";
    first = false;
    buf << id;
  }
  return buf.str();
}

} // namespace crispBis
