#include "SuperBlock.h"
#include "SuperPartition.h"
#include "Block.h"
#include "../lanTools.h"
#include <set>

namespace crispBis {

SuperBlock::SuperBlock(SuperPartition* superPartition) {
  this->superPartition = superPartition;
  superPartition->addSuperBlock(this);
}

Block* SuperBlock::smallerBlock() {
  assert(compound());
  size_t n = blocks.size();
  Block *b1 = blocks[n-1];
  Block *b2 = blocks[n-2];
  if (b1->size() < b2->size())
    return b1;
  return b2;
}

void SuperBlock::addBlock(Block *b) {
  blocks.push_back(b);
  if (blocks.size() == 2) {
    superPartition->simpleSuperBlocks->remove(this);
    superPartition->compoundSuperBlocks->add(this);
  }
}

void SuperBlock::removeBlock(Block *b) {
  size_t n = blocks.size();
  assert(n >= 2 && (b == blocks[n-1] || b == blocks[n-2]));

  if (b == blocks[n-1])
    blocks.pop_back();
  else if (b == blocks[n-2]) {
    Block *b2 = blocks[n-1];
    blocks.pop_back();
    blocks.pop_back();
    blocks.push_back(b2);
  }

  if (blocks.size() == 1) {
    superPartition->compoundSuperBlocks->remove(this);
    superPartition->simpleSuperBlocks->add(this);
  }
}

void SuperBlock::createSuperBlock(SuperPartition *sp, Block *b, size_t r)
{
  SuperBlock *sb = NEW(SuperBlock, SuperBlock(sp));
  sb->addBlock(b);
  b->superBlocks[r] = sb;
}

std::string SuperBlock::str() {
  std::set<std::string> _blocks;
  for (Block *b: blocks)
    _blocks.insert(b->str());

  std::stringstream buf;
  buf << "{";
  bool first = true;
  for (const std::string &b: _blocks) {
    if (!first)
      buf << ", ";
    first = false;
    buf << b;
  }
  buf << "}";
  return buf.str();
}

} // namespace crispBis
