#include "../lanTools.h"
#include "Vertex.h"
#include "Edge.h"
#include "CrispBis.h"
#include <string>
#include <sstream>

namespace crispBis {

Vertex::~Vertex() {
  for (auto &x: comingEdges) {
    for (Edge *e: x.second)
      DELETE(e);
  }
}

Vertex* Vertex::getVertex(CrispBis *crispBisInst, const std::string ID) {
  Vertex *p = crispBisInst->vertices.getPtr(ID);
  if (!p->crispBisInst)
    p->crispBisInst = crispBisInst;

  assert(p->crispBisInst == crispBisInst);
  return p;
}

void Vertex::addLabel(std::string p, float degree) {
  size_t vertexLabelID = crispBisInst->vertexLabels.getID(p);
  if (label.find(vertexLabelID) != label.end()) {
    std::stringstream buf;
    buf << "Duplicated label " << p << " for node " << getID() << ".";
    throw buf.str();
  }
  label[vertexLabelID] = degree;
}

std::string Vertex::getID() {
  return crispBisInst->vertices.getName(this);
}

std::string Vertex::str() {
  std::stringstream buf;
  buf.precision(3);
  buf << "vertex(" << getID() << ": {";
  std::map<std::string, float> _label;
  for (auto &x: label) {
    size_t id = x.first;
    _label[crispBisInst->vertexLabels.getName(id)] = x.second;
  }
  bool first = true;
  for (auto &x: label) {
    if (!first)
      buf << ", ";
    first = false;
    buf << x.first << ":" << x.second;
  }
  buf << "})";
  return buf.str();
}

} // namespace crispBis
