#include <cstdlib>
#include <string>
#include <sstream>
#include <set>
#include <tuple>
#include <time.h>
#include "tests.h"
#include "CrispBis.h"
#include "DLList.h"
#include "lanTools.h"
#include "Vertex.h"
#include "Edge.h"

namespace crispBis {

//------------------------------------------------------------------------------

struct TestDLListNode {
  int value;
  TestDLListNode *next = nullptr;
  TestDLListNode *prev = nullptr;

  TestDLListNode(int value) : value(value) {};
  ~TestDLListNode() {};

  std::string str() { return std::to_string(value); };
};

void Test::testDLList() {
  TestDLListNode* v1 = NEW(TestDLListNode, TestDLListNode(10));
  TestDLListNode* v2 = NEW(TestDLListNode, TestDLListNode(20));
  TestDLListNode* v3 = NEW(TestDLListNode, TestDLListNode(30));
  TestDLListNode* v4 = NEW(TestDLListNode, TestDLListNode(40));
  TestDLListNode* v5 = NEW(TestDLListNode, TestDLListNode(50));

  DLList<TestDLListNode> lista(false);
  lista.add(v5);
  lista.add(v4);
  lista.add(v3);
  lista.add(v2);
  lista.add(v1);
  assert(lista.str() == std::string("[10, 20, 30, 40, 50]"));
  lista.remove(v3);
  assert(lista.str() == std::string("[10, 20, 40, 50]"));
  lista.remove(v1);
  assert(lista.str() == std::string("[20, 40, 50]"));
  lista.remove(v5);
  assert(lista.str() == std::string("[20, 40]"));
  lista.remove(v2);
  assert(lista.str() == std::string("[40]"));
  lista.add(v1);
  assert(lista.str() == std::string("[10, 40]"));
  lista.remove(v4);
  assert(lista.str() == std::string("[10]"));
  lista.remove(v1);
  assert(lista.str() == std::string("[]"));

  DELETE(v1);
  DELETE(v2);
  DELETE(v3);
  DELETE(v4);
  DELETE(v5);

  TestDLListNode* v6 = NEW(TestDLListNode, TestDLListNode(50));
  DLList<TestDLListNode> lista2(true);
  lista2.add(v6);
  // to check that v6 will be freed at the end
}

//------------------------------------------------------------------------------

struct Scenario {
  int N,g,n,m,p,l,nSV,nSE;
};

void Test::testPerformance() {
  srand(time(NULL));
  Scenario scenarios[] = {
    /*
    {2,100,10,30,5,3,1,1},
    {2,100,10,30,5,3,2,1},
    {2,100,10,30,5,3,10,1},
    {2,100,10,30,5,3,1,2},
    {2,100,10,30,5,3,1,10},
    {2,100,10,30,5,3,10,10},
    {2,100,10,40,5,3,1,1},
    {2,100,10,50,5,3,1,1},
    {2,100,10,60,5,3,1,1},

    {2,100,10,30,5,10,1,1},
    {2,100,10,30,5,10,2,1},
    {2,100,10,30,5,10,10,1},
    {2,100,10,30,5,10,1,2},
    {2,100,10,30,5,10,1,10},
    {2,100,10,30,5,10,10,10},
    {2,100,10,40,5,10,1,1},
    {2,100,10,50,5,10,1,1},
    {2,100,10,60,5,10,1,1},
    */

    {2,1000,10,30,5,3,1,1},
    {2,1000,10,30,5,3,2,1},
    {2,1000,10,30,5,3,10,1},
    {2,1000,10,30,5,3,1,2},
    {2,1000,10,30,5,3,1,10},
    {2,1000,10,30,5,3,10,10},
    {2,1000,100,400,50,3,1,1},
    {2,1000,100,500,50,3,1,1},
    {2,1000,100,600,50,3,1,1},
    {2,1000,100,700,50,3,1,1},
    {2,1000,100,800,50,3,1,1},
    {2,1000,100,900,50,3,1,1},
    {2,1000,100,1000,50,3,1,1},

    {2,1000,10,30,5,10,1,1},
    {2,1000,10,30,5,10,2,1},
    {2,1000,10,30,5,10,10,1},
    {2,1000,10,30,5,10,1,2},
    {2,1000,10,30,5,10,1,10},
    {2,1000,10,30,5,10,10,10},
    {2,1000,100,400,50,10,1,1},
    {2,1000,100,500,50,10,1,1},
    {2,1000,100,600,50,10,1,1},
    {2,1000,100,700,50,10,1,1},
    {2,1000,100,800,50,10,1,1},
    {2,1000,100,900,50,10,1,1},
    {2,1000,100,1000,50,10,1,1},

    {2,1000,10,30,5,100,1,1},
    {2,1000,10,30,5,100,2,1},
    {2,1000,10,30,5,100,10,1},
    {2,1000,10,30,5,100,1,2},
    {2,1000,10,30,5,100,1,10},
    {2,1000,10,30,5,100,10,10},
    {2,1000,100,400,50,100,1,1},
    {2,1000,100,500,50,100,1,1},
    {2,1000,100,600,50,100,1,1},
    {2,1000,100,700,50,100,1,1},
    {2,1000,100,800,50,100,1,1},
    {2,1000,100,900,50,100,1,1},
    {2,1000,100,1000,50,100,1,1},
  };

  for (auto s: scenarios) {
    int N = s.N, g = s.g, n = s.n, m = s.m, p = s.p, l = s.l, nSV = s.nSV, nSE = s.nSE;
    assert(l >= 2);
    char buf[512];
    sprintf(buf, "TESTING SCENARIO [N,g,n,m,p,l,nSV,nSE] = [%d,%d,%d,%d,%d,%d,%d,%d]\n",
            N,g,n,m,p,l,nSV,nSE);
    std::cout << buf;

    for (int k = 0; k < N; k++) {
      CrispBis *inst = NEW(CrispBis, CrispBis(false));

      for (int i = 0; i < g; i++) {
        std::set<std::pair<size_t, size_t>> vertexLabels;
        while (vertexLabels.size() < size_t(p)) {
          int x = i*n + rand() % n;
          int q = rand() % nSV;
          vertexLabels.insert(std::make_pair(x,q));
        }
        for (auto &it: vertexLabels) {
          float d = (rand() % l + 1) * 1.0 / l;
          inst->addVertexLabel(std::to_string(it.first),
                               std::to_string(it.second), d);
        }

        std::set<std::tuple<size_t, size_t, size_t>> edges;
        while (edges.size() < size_t(m)) {
          int x = i*n + rand() % n;
          int y = i*n + rand() % n;
          int r = rand() % nSE;
          edges.insert(std::make_tuple(x,y,r));
        }
        for (auto &it: edges) {
          float d = (rand() % l + 1) * 1.0 / l;
          inst->addEdge(std::to_string(std::get<0>(it)),
                        std::to_string(std::get<1>(it)),
                        std::to_string(std::get<2>(it)), d);

        }
      } // for i

      clock_t t1 = clock();
      inst->compCB();
      int size = inst->P.size();
      DELETE(inst);
      CHECK_MEMORY();
      clock_t t2 = clock();
      double elapsed = double(t2 - t1)/CLOCKS_PER_SEC;
      std::cout << "Done a random test #" << k+1 << ": "
                << elapsed << "s, " << size << " blocks.\n";
    }
  }
}

static std::string x_i_j(const char *s, size_t i, size_t j) {
  return std::string(s) + "_" + std::to_string(i) + "_" + std::to_string(j);
}

void Test::testCorrectness1(const size_t k, bool cyclic) {
  assert(k > 0);
  CrispBis *inst = NEW(CrispBis, CrispBis(false));

  float d1, d2;
  int dra, dsa, drb, dsb;

  d1 = (rand() % 100 + 1) / 100.0;
  do {
    d2 = (rand() % 100 + 1) / 100.0;
  } while (d2 == d1);

  dra = rand() % k + 1;
  dsa = rand() % k + 1;
  drb = rand() % k + 1;
  dsb = rand() % k + 1;

  for (size_t i = 0; i < k; i++) {
    size_t ni = (i + 1 < k ? i + 1 : 0);

    for (size_t j = 0; j < k; j++) {
      inst->addVertexLabel(x_i_j("a", i, j), "p", d1);
      inst->addVertexLabel(x_i_j("b", i, j), "p", d2);
      if (cyclic && i == 0) {
        inst->addVertexLabel(x_i_j("a", i, j), "q", 1);
        inst->addVertexLabel(x_i_j("b", i, j), "q", 1);
      }

      size_t taa = rand() % k;
      size_t tab = rand() % k;
      size_t tba = rand() % k;
      size_t tbb = rand() % k;

      if (i + 1 < k || cyclic) {
        for (size_t jp = 0; jp < k; jp++) {
#ifndef COUNTING_SUCCESSORS
          size_t rdr = rand(), rds = rand();
#else
          size_t rdr = 1, rds = 1;
#endif // COUNTING_SUCCESSORS

          if (jp == taa) {
            inst->addEdge(x_i_j("a", i, j), x_i_j("a", ni, jp), "r",
                          1.0 * dra / k);
            inst->addEdge(x_i_j("a", i, j), x_i_j("a", ni, jp), "s",
                          1.0 * dsa / k);
          }
          else {
            inst->addEdge(x_i_j("a", i, j), x_i_j("a", ni, jp), "r",
                          1.0 * (dra - rdr % dra) / k);
            inst->addEdge(x_i_j("a", i, j), x_i_j("a", ni, jp), "s",
                          1.0 * (dsa - rds % dsa) / k);
          }

          if (jp == tab) {
            inst->addEdge(x_i_j("a", i, j), x_i_j("b", ni, jp), "r",
                          1.0 * drb / k);
            inst->addEdge(x_i_j("a", i, j), x_i_j("b", ni, jp), "s",
                          1.0 * dsb / k);
          }
          else {
            inst->addEdge(x_i_j("a", i, j), x_i_j("b", ni, jp), "r",
                          1.0 * (drb - rdr % drb) / k);
            inst->addEdge(x_i_j("a", i, j), x_i_j("b", ni, jp), "s",
                          1.0 * (dsb - rds % dsb) / k);
          }

          if (jp == tba) {
            inst->addEdge(x_i_j("b", i, j), x_i_j("a", ni, jp), "r",
                          1.0 * dra / k);
            inst->addEdge(x_i_j("b", i, j), x_i_j("a", ni, jp), "s",
                          1.0 * dsa / k);
          }
          else {
            inst->addEdge(x_i_j("b", i, j), x_i_j("a", ni, jp), "r",
                          1.0 * (dra - rdr % dra) / k);
            inst->addEdge(x_i_j("b", i, j), x_i_j("a", ni, jp), "s",
                          1.0 * (dsa - rds % dsa) / k);
          }

          if (jp == tbb) {
            inst->addEdge(x_i_j("b", i, j), x_i_j("b", ni, jp), "r",
                          1.0 * drb / k);
            inst->addEdge(x_i_j("b", i, j), x_i_j("b", ni, jp), "s",
                          1.0 * dsb / k);
          }
          else {
            inst->addEdge(x_i_j("b", i, j), x_i_j("b", ni, jp), "r",
                          1.0 * (drb - rdr % drb) / k);
            inst->addEdge(x_i_j("b", i, j), x_i_j("b", ni, jp), "s",
                          1.0 * (dsb - rds % dsb) / k);
          }
        } // for jp
      } // if
    } // for j
  } // for i

  clock_t t1 = clock();
  inst->compCB();
  clock_t t2 = clock();
  double elapsed = double(t2 - t1)/CLOCKS_PER_SEC;
  std::cout << "Elapsed time: " << elapsed << "s.\n";
  std::string rs = inst->P.str();
  DELETE(inst);
  CHECK_MEMORY();

  std::set<std::string> ans_set;
  for (size_t i = 0; i < k; i++) {
    std::set<std::string> a_i_set, b_i_set;
    for (size_t j = 0; j < k; j++) {
      a_i_set.insert(x_i_j("a", i, j));
      b_i_set.insert(x_i_j("b", i, j));
    }

    std::set<std::string>* tab[] = {&a_i_set, &b_i_set};
    for (auto zb: tab) {
      std::stringstream buf;
      bool first = true;
      for (auto &x: *zb) {
        if (!first)
          buf << " ";
        first = false;
        buf << x;
      }
      ans_set.insert(buf.str());
    }
  }

  std::stringstream buf;
  bool first = true;
  for (auto &x: ans_set) {
    if (!first)
      buf << "\n";
    first = false;
    buf << x;
  }
  std::string answer = buf.str();

  ASSERT(rs == answer);
}

void Test::testCorrectness2(const size_t k) {
  assert(k > 0);
  CrispBis *inst = NEW(CrispBis, CrispBis(false));

  for (size_t i = 0; i < k; i++) {
    size_t ni = (i + 1 < k ? i + 1 : 0);
    for (size_t j = 0; j < k; j++) {
      const char* ab[] = {"a", "b"};
      for (auto c: ab) {
        if (j == 0)
          inst->addVertexLabel(x_i_j(c, i, j), "p", 1);

        size_t nj = (j + 1 < k ? j + 1 : 0);
        for (size_t ip = 0; ip < k; ip++) {
          if ((ip == i && j < k - 1) || (ip == ni && j == k - 1)) {
            inst->addEdge(x_i_j(c, i, j), x_i_j(c, ip, nj), "r", 1);
            inst->addEdge(x_i_j(c, i, j), x_i_j(c, ip, nj), "s", 1);
          }
          else {
#ifndef COUNTING_SUCCESSORS
            inst->addEdge(x_i_j(c, i, j), x_i_j(c, ip, nj), "r",
                          1.0 * (rand() % k + 1) / k);
            inst->addEdge(x_i_j(c, i, j), x_i_j(c, ip, nj), "s",
                          1.0 * (rand() % k + 1) / k);
#else
            inst->addEdge(x_i_j(c, i, j), x_i_j(c, ip, nj), "r", 1.0 / k);
            inst->addEdge(x_i_j(c, i, j), x_i_j(c, ip, nj), "s", 1.0 / k);
#endif // COUNTING_SUCCESSORS
          }
        } // for ip
      } // for c
    } // for j
  } // for i

  clock_t t1 = clock();
  inst->compCB();
  clock_t t2 = clock();
  double elapsed = double(t2 - t1)/CLOCKS_PER_SEC;
  std::cout << "Elapsed time: " << elapsed << "s.\n";
  std::string rs = inst->P.str();
  DELETE(inst);
  CHECK_MEMORY();

  std::set<std::string> ans_set;
  for (size_t j = 0; j < k; j++) {
    std::set<std::string> zb;
    for (size_t i = 0; i < k; i++) {
      zb.insert(x_i_j("a", i, j));
      zb.insert(x_i_j("b", i, j));
    }

    std::stringstream buf;
    bool first = true;
    for (auto &x: zb) {
      if (!first)
        buf << " ";
      first = false;
      buf << x;
    }
    ans_set.insert(buf.str());
  } // for j

  std::stringstream buf;
  bool first = true;
  for (auto &x: ans_set) {
    if (!first)
      buf << "\n";
    first = false;
    buf << x;
  }
  std::string answer = buf.str();

  ASSERT(rs == answer);
}

//------------------------------------------------------------------------------

} // namespace crispBis
